@@ -50,7 +50,10 @@ mpfr_get_flt (mpfr_srcptr src, mpfr_rnd_t rnd_mode)
     rnd_mode = negative ? MPFR_RNDD : MPFR_RNDU;
 
   /* FIXME: The code below assumes the IEEE-754 binary32 format
-     with subnormal support. */
+     with subnormal support. Fix it by converting to double, then
+     to float, and in case of binary radix (for which we want
+     correct rounding), handle double-rounding issues somewhere
+     in the code? */
 
   /* the smallest positive normal float number is 2^(-126) = 0.5*2^(-125),
      and the smallest positive subnormal number is 2^(-149) = 0.5*2^(-148) */
