@@ -1,6 +1,6 @@
 /* Memory allocation used during tests.
 
-Copyright 2001-2003, 2006-2016 Free Software Foundation, Inc.
+Copyright 2001-2003, 2006-2017 Free Software Foundation, Inc.
 Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
@@ -22,10 +22,6 @@ http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 
 /* Note: this file comes from GMP's tests/memory.c */
 
-#include <stdio.h>
-#include <stdlib.h>  /* for abort */
-#include <limits.h>
-
 #include "mpfr-test.h"
 
 /* Each block allocated is a separate malloc, for the benefit of a redzoning
@@ -38,16 +34,72 @@ http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
    when tests_memory_end() is called.  Test programs must be sure to have
    "clear"s for all temporary variables used.  */
 
+/* Note about error messages
+   -------------------------
+   Error messages in MPFR are usually written to stdout. However, those
+   coming from the memory allocator need to be written to stderr in order
+   to be visible when the standard output is redirected, e.g. in the tests
+   of I/O functions (like tprintf). For consistency, all error messages in
+   this file should be written to stderr. */
+
 struct header {
   void           *ptr;
   size_t         size;
   struct header  *next;
 };
 
+/* The memory limit can be changed with the MPFR_TESTS_MEMORY_LIMIT
+   environment variable. This is normally not necessary (a failure
+   would mean a bug), thus not recommended, for "make check". But
+   some test programs can take arguments for particular tests, which
+   may need more memory. This variable is exported, so that such
+   programs may also change the memory limit. */
+size_t tests_memory_limit = DEFAULT_MEMORY_LIMIT;
+
 static struct header  *tests_memory_list;
+static size_t tests_total_size = 0;
+MPFR_LOCK_DECL(mpfr_lock_memory)
+
+static void *
+mpfr_default_allocate (size_t size)
+{
+  void *ret;
+  ret = malloc (size);
+  if (MPFR_UNLIKELY (ret == NULL))
+    {
+      fprintf (stderr, "[MPFR] mpfr_default_allocate(): "
+               "can't allocate memory (size=%lu)\n",
+               (unsigned long) size);
+      abort ();
+    }
+  return ret;
+}
+
+static void *
+mpfr_default_reallocate (void *oldptr, size_t old_size, size_t new_size)
+{
+  void *ret;
+  ret = realloc (oldptr, new_size);
+  if (MPFR_UNLIKELY(ret == NULL))
+    {
+      fprintf (stderr, "[MPFR] mpfr_default_reallocate(): "
+               "can't reallocate memory (old_size=%lu new_size=%lu)\n",
+               (unsigned long) old_size, (unsigned long) new_size);
+      abort ();
+    }
+  return ret;
+}
+
+static void
+mpfr_default_free (void *blk_ptr, size_t blk_size)
+{
+  free (blk_ptr);
+}
 
 /* Return a pointer to a pointer to the found block (so it can be updated
    when unlinking). */
+/* FIXME: This is a O(n) search, while it could be done in nearly
+   constant time with a better data structure! */
 static struct header **
 tests_memory_find (void *ptr)
 {
@@ -68,23 +120,46 @@ tests_memory_valid (void *ptr)
 }
 */
 
+static void
+tests_addsize (size_t size)
+{
+  tests_total_size += size;
+  if (tests_total_size > tests_memory_limit)
+    {
+      /* The total size taken by MPFR on the heap is more than 4 MB:
+         either a bug or a huge inefficiency. */
+      fprintf (stderr, "[MPFR] tests_addsize(): "
+               "too much memory (%lu bytes)\n",
+              (unsigned long) tests_total_size);
+      abort ();
+    }
+}
+
 void *
 tests_allocate (size_t size)
 {
   struct header  *h;
 
+  MPFR_LOCK_WRITE(mpfr_lock_memory);
+
   if (size == 0)
     {
-      printf ("tests_allocate(): attempt to allocate 0 bytes\n");
+      fprintf (stderr, "[MPFR] tests_allocate(): "
+               "attempt to allocate 0 bytes\n");
       abort ();
     }
 
-  h = (struct header *) __gmp_default_allocate (sizeof (*h));
+  tests_addsize (size);
+
+  h = (struct header *) mpfr_default_allocate (sizeof (*h));
   h->next = tests_memory_list;
   tests_memory_list = h;
 
   h->size = size;
-  h->ptr = __gmp_default_allocate (size);
+  h->ptr = mpfr_default_allocate (size);
+
+  MPFR_UNLOCK_WRITE(mpfr_lock_memory);
+
   return h->ptr;
 }
 
@@ -93,9 +168,12 @@ tests_reallocate (void *ptr, size_t old_size, size_t new_size)
 {
   struct header  **hp, *h;
 
+  MPFR_LOCK_WRITE(mpfr_lock_memory);
+
   if (new_size == 0)
     {
-      printf ("tests_reallocate(): attempt to reallocate 0x%lX to 0 bytes\n",
+      fprintf (stderr, "[MPFR] tests_reallocate(): "
+               "attempt to reallocate 0x%lX to 0 bytes\n",
               (unsigned long) ptr);
       abort ();
     }
@@ -103,7 +181,8 @@ tests_reallocate (void *ptr, size_t old_size, size_t new_size)
   hp = tests_memory_find (ptr);
   if (hp == NULL)
     {
-      printf ("tests_reallocate(): attempt to reallocate bad pointer 0x%lX\n",
+      fprintf (stderr, "[MPFR] tests_reallocate(): "
+               "attempt to reallocate bad pointer 0x%lX\n",
               (unsigned long) ptr);
       abort ();
     }
@@ -113,13 +192,20 @@ tests_reallocate (void *ptr, size_t old_size, size_t new_size)
     {
       /* Note: we should use the standard %zu to print sizes, but
          this is not supported by old C implementations. */
-      printf ("tests_reallocate(): bad old size %lu, should be %lu\n",
+      fprintf (stderr, "[MPFR] tests_reallocate(): "
+               "bad old size %lu, should be %lu\n",
               (unsigned long) old_size, (unsigned long) h->size);
       abort ();
     }
 
+  tests_total_size -= old_size;
+  tests_addsize (new_size);
+
   h->size = new_size;
-  h->ptr = __gmp_default_reallocate (ptr, old_size, new_size);
+  h->ptr = mpfr_default_reallocate (ptr, old_size, new_size);
+
+  MPFR_UNLOCK_WRITE(mpfr_lock_memory);
+
   return h->ptr;
 }
 
@@ -129,7 +215,8 @@ tests_free_find (void *ptr)
   struct header  **hp = tests_memory_find (ptr);
   if (hp == NULL)
     {
-      printf ("tests_free(): attempt to free bad pointer 0x%lX\n",
+      fprintf (stderr, "[MPFR] tests_free(): "
+               "attempt to free bad pointer 0x%lX\n",
               (unsigned long) ptr);
       abort ();
     }
@@ -144,33 +231,51 @@ tests_free_nosize (void *ptr)
 
   *hp = h->next;  /* unlink */
 
-  __gmp_default_free (ptr, h->size);
-  __gmp_default_free (h, sizeof (*h));
+  mpfr_default_free (ptr, h->size);
+  mpfr_default_free (h, sizeof (*h));
 }
 
 void
 tests_free (void *ptr, size_t size)
 {
-  struct header  **hp = tests_free_find (ptr);
-  struct header  *h = *hp;
+  struct header  **hp;
+  struct header  *h;
+
+  MPFR_LOCK_WRITE(mpfr_lock_memory);
+
+  hp = tests_free_find (ptr);
+  h = *hp;
 
   if (h->size != size)
     {
       /* Note: we should use the standard %zu to print sizes, but
          this is not supported by old C implementations. */
-      printf ("tests_free(): bad size %lu, should be %lu\n",
+      fprintf (stderr, "[MPFR] tests_free(): bad size %lu, should be %lu\n",
               (unsigned long) size, (unsigned long) h->size);
       abort ();
     }
 
+  tests_total_size -= size;
   tests_free_nosize (ptr);
+
+  MPFR_UNLOCK_WRITE(mpfr_lock_memory);
 }
 
 void
 tests_memory_start (void)
 {
+  char *p;
+
   tests_memory_list = NULL;
   mp_set_memory_functions (tests_allocate, tests_reallocate, tests_free);
+
+  p = getenv ("MPFR_TESTS_MEMORY_LIMIT");
+  if (p != NULL)
+    {
+      tests_memory_limit = strtoul (p, NULL, 0);
+      if (tests_memory_limit == 0)
+        tests_memory_limit = (size_t) -1;  /* no memory limit */
+    }
 }
 
 void
@@ -181,13 +286,13 @@ tests_memory_end (void)
       struct header  *h;
       unsigned  count;
 
-      printf ("tests_memory_end(): not all memory freed\n");
+      fprintf (stderr, "[MPFR] tests_memory_end(): not all memory freed\n");
 
       count = 0;
       for (h = tests_memory_list; h != NULL; h = h->next)
         count++;
 
-      printf ("    %u blocks remaining\n", count);
+      fprintf (stderr, "[MPFR]    %u blocks remaining\n", count);
       abort ();
     }
 }
