@@ -1,6 +1,6 @@
 /* Test file for mpfr_can_round and mpfr_round_p.
 
-Copyright 1999, 2001-2016 Free Software Foundation, Inc.
+Copyright 1999, 2001-2017 Free Software Foundation, Inc.
 Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
@@ -20,9 +20,6 @@ along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
 http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
-#include <stdio.h>
-#include <stdlib.h>
-
 #include "mpfr-test.h"
 
 /* Simple cases where err - prec is large enough.
@@ -41,6 +38,8 @@ test_simple (void)
           mpfr_t b;
           int p, err, prec, inex, c;
 
+          if (r2 == MPFR_RNDF)
+            continue;
           p = 12 + (randlimb() % (2 * GMP_NUMB_BITS));
           err = p - 3;
           prec = 4;
@@ -49,8 +48,9 @@ test_simple (void)
           MPFR_ASSERTN (inex == 0);
           c = mpfr_can_round (b, err, (mpfr_rnd_t) r1, (mpfr_rnd_t) r2, prec);
           /* If r1 == r2, we can round.
+             Note: For r1, RNDF is equivalent to RNDN.
              TODO: complete this test for r1 != r2. */
-          if (r1 == r2 && !c)
+          if ((r1 == MPFR_RNDF ? MPFR_RNDN : r1) == r2 && !c)
             {
               printf ("Error in test_simple for i=%d,"
                       " err=%d r1=%s, r2=%s, p=%d\n", i, err,
@@ -91,7 +91,9 @@ check_round_p (void)
           printf ("mpfr_round_p(%d) != mpfr_can_round(%d)!\n"
                   "bn = %ld, err0 = %ld, prec = %lu\nbp = ",
                   r1, r2, n, (long) err, (unsigned long) p);
+#ifndef MPFR_USE_MINI_GMP
           gmp_printf ("%NX\n", buf, n);
+#endif
           exit (1);
         }
     }
@@ -107,6 +109,8 @@ test_pow2 (mpfr_exp_t i, mpfr_prec_t px, mpfr_rnd_t r1, mpfr_rnd_t r2,
 
   mpfr_init2 (x, px);
   mpfr_set_ui_2exp (x, 1, i, MPFR_RNDN);
+  /* for mpfr_can_round, r1=RNDF is equivalent to r1=RNDN (the sign of the
+     error is not known) */
   b = !!mpfr_can_round (x, i+1, r1, r2, prec);
   /* Note: If mpfr_can_round succeeds for both
      (r1,r2) = (MPFR_RNDD,MPFR_RNDN) and
@@ -197,6 +201,8 @@ check_can_round (void)
                   RND_LOOP (rnd1)
                     RND_LOOP (rnd2)
                       {
+                        if (rnd2 == MPFR_RNDF)
+                          continue;
                         mpfr_set (yinf, MPFR_IS_LIKE_RNDD (rnd1, 1) ?
                                   x : xinf, (mpfr_rnd_t) rnd2);
                         mpfr_set (ysup, MPFR_IS_LIKE_RNDU (rnd1, 1) ?
@@ -241,14 +247,14 @@ main (void)
      bit to zero in case of equal distance */
   mpfr_init2 (x, 59);
   mpfr_set_str_binary (x, "-0.10010001010111000011110010111010111110000000111101100111111E663");
-  if (mpfr_can_round (x, 54, MPFR_RNDZ, MPFR_RNDZ, 53) != 0)
+  if (mpfr_can_round (x, 54, MPFR_RNDZ, MPFR_RNDZ, 53))
     {
       printf ("Error (1) in mpfr_can_round\n");
       exit (1);
     }
 
   mpfr_set_str_binary (x, "-Inf");
-  if (mpfr_can_round (x, 2000, MPFR_RNDZ, MPFR_RNDZ, 2000) != 0)
+  if (mpfr_can_round (x, 2000, MPFR_RNDZ, MPFR_RNDZ, 2000))
     {
       printf ("Error (2) in mpfr_can_round\n");
       exit (1);
@@ -264,7 +270,7 @@ main (void)
 
   mpfr_set_prec (x, 137);
   mpfr_set_str_binary (x, "-0.10111001101001010110011000110100111010011101101010010100101100001110000100111111011101010110001010111100100101110111100001000010000000000E-97");
-  if (mpfr_can_round (x, 132, MPFR_RNDU, MPFR_RNDZ, 128) == 0)
+  if (! mpfr_can_round (x, 132, MPFR_RNDU, MPFR_RNDZ, 128))
     {
       printf ("Error (4) in mpfr_can_round\n");
       exit (1);
@@ -273,7 +279,7 @@ main (void)
   /* in the following, we can round but cannot determine the inexact flag */
   mpfr_set_prec (x, 86);
   mpfr_set_str_binary (x, "-0.11100100010011001111011010100111101010011000000000000000000000000000000000000000000000E-80");
-  if (mpfr_can_round (x, 81, MPFR_RNDU, MPFR_RNDZ, 44) == 0)
+  if (! mpfr_can_round (x, 81, MPFR_RNDU, MPFR_RNDZ, 44))
     {
       printf ("Error (5) in mpfr_can_round\n");
       exit (1);
@@ -283,12 +289,13 @@ main (void)
   mpfr_set_str (x, "0.ff4ca619c76ba69", 16, MPFR_RNDZ);
   for (i = 30; i < 99; i++)
     for (j = 30; j < 99; j++)
-      for (r1 = 0; r1 < MPFR_RND_MAX; r1++)
-        for (r2 = 0; r2 < MPFR_RND_MAX; r2++)
-          {
-            /* test for assertions */
-            mpfr_can_round (x, i, (mpfr_rnd_t) r1, (mpfr_rnd_t) r2, j);
-          }
+      RND_LOOP (r1)
+        RND_LOOP (r2)
+          if (r2 != MPFR_RNDF)
+            {
+              /* test for assertions */
+              mpfr_can_round (x, i, (mpfr_rnd_t) r1, (mpfr_rnd_t) r2, j);
+            }
 
   test_pow2 (32, 32, MPFR_RNDN, MPFR_RNDN, 32);
   test_pow2 (174, 174, MPFR_RNDN, MPFR_RNDN, 174);
@@ -302,10 +309,11 @@ main (void)
     {
       i = (randlimb() % 200) + 4;
       for (j = i - 2; j < i + 2; j++)
-        for (r1 = 0; r1 < MPFR_RND_MAX; r1++)
-          for (r2 = 0; r2 < MPFR_RND_MAX; r2++)
-            for (k = MPFR_PREC_MIN; k <= i + 2; k++)
-              test_pow2 (i, k, (mpfr_rnd_t) r1, (mpfr_rnd_t) r2, j);
+        RND_LOOP (r1)
+          RND_LOOP (r2)
+            if (r2 != MPFR_RNDF)
+              for (k = MPFR_PREC_MIN; k <= i + 2; k++)
+                test_pow2 (i, k, (mpfr_rnd_t) r1, (mpfr_rnd_t) r2, j);
     }
 
   mpfr_clear (x);
