@@ -1,6 +1,6 @@
 /* Test file for mpfr_const_log2.
 
-Copyright 1999, 2001-2016 Free Software Foundation, Inc.
+Copyright 1999, 2001-2017 Free Software Foundation, Inc.
 Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
@@ -20,9 +20,6 @@ along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
 http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
-#include <stdio.h>
-#include <stdlib.h>
-
 #include "mpfr-test.h"
 
 /* tlog2 [prec] [rnd] [0 = no print] */
@@ -31,8 +28,7 @@ static void
 check (mpfr_prec_t p0, mpfr_prec_t p1)
 {
   mpfr_t x, y, z;
-  mpfr_rnd_t rnd;
-  int dif;
+  int i, inex, inex_ref;
 
   mpfr_init (x);
   mpfr_init (y);
@@ -40,22 +36,28 @@ check (mpfr_prec_t p0, mpfr_prec_t p1)
   mpfr_const_log2 (z, MPFR_RNDN);
   mpfr_clear_cache (__gmpfr_cache_const_log2);
 
-  for (; p0<=p1; p0++)
+  for (; p0 <= p1; p0++)
     {
       mpfr_set_prec (x, p0);
       mpfr_set_prec (y, p0);
+      RND_LOOP_NO_RNDF (i)
         {
-          rnd = RND_RAND ();
-          mpfr_const_log2 (x, rnd);
-          mpfr_set (y, z, rnd);
-          if ((dif = mpfr_cmp (x, y))
-              && mpfr_can_round (z, mpfr_get_prec(z), MPFR_RNDN,
-                                                 rnd, p0))
+          mpfr_rnd_t rnd = (mpfr_rnd_t) i;
+          inex = mpfr_const_log2 (x, rnd);
+          inex_ref = mpfr_set (y, z, rnd);
+          if (! mpfr_can_round (z, mpfr_get_prec (z), MPFR_RNDN, rnd, p0))
             {
-              printf ("mpfr_const_log2 fails for prec=%u, rnd=%s Diff=%d\n",
-                      (unsigned int) p0, mpfr_print_rnd_mode (rnd), dif);
+              printf ("increase guard precision in check()\n");
+              exit (1);
+            }
+          if (mpfr_cmp (x, y) || inex != inex_ref)
+            {
+              printf ("mpfr_const_log2 fails for prec=%u, rnd=%s\n",
+                      (unsigned int) p0, mpfr_print_rnd_mode (rnd));
               printf ("expected "), mpfr_dump (y);
               printf ("got      "), mpfr_dump (x);
+              printf ("expected inex = %d\n", inex_ref);
+              printf ("got      inex = %d\n", inex);
               exit (1);
             }
         }
@@ -69,15 +71,27 @@ check (mpfr_prec_t p0, mpfr_prec_t p1)
 static void
 check_large (void)
 {
-  mpfr_t x, y;
+  mpfr_t x, y, z;
+
   mpfr_init2 (x, 25000);
   mpfr_init2 (y, 26000);
+  mpfr_init2 (z, 26000);
   (mpfr_const_log2) (x, MPFR_RNDN); /* First one ! */
   (mpfr_const_log2) (y, MPFR_RNDN); /* Then the other - cache - */
+  mpfr_set (z, y, MPFR_RNDN);
   mpfr_prec_round (y, 25000, MPFR_RNDN);
-  if (mpfr_cmp (x, y))
+  if (mpfr_cmp (x, y) != 0)
     {
       printf ("const_log2: error for large prec\n");
+      printf ("x = ");
+      mpfr_out_str (stdout, 16, 0, x, MPFR_RNDN);
+      printf ("\n");
+      printf ("y = ");
+      mpfr_out_str (stdout, 16, 0, y, MPFR_RNDN);
+      printf ("\n");
+      printf ("z = ");
+      mpfr_out_str (stdout, 16, 0, z, MPFR_RNDN);
+      printf ("\n");
       exit (1);
     }
 
@@ -86,7 +100,7 @@ check_large (void)
   mpfr_set_prec (x, 26249);
   mpfr_const_log2 (x, MPFR_RNDZ);
 
-  mpfr_clears (x, y, (mpfr_ptr) 0);
+  mpfr_clears (x, y, z, (mpfr_ptr) 0);
 }
 
 static void
@@ -146,11 +160,13 @@ main (int argc, char *argv[])
   tests_start_mpfr ();
 
   p = (argc>1) ? atoi(argv[1]) : 53;
-  rnd = (argc>2) ? (mpfr_rnd_t) atoi(argv[2]) : MPFR_RNDZ;
+  rnd = (argc>2) ? (mpfr_rnd_t) atoi(argv[2]) : MPFR_RNDN;
 
   mpfr_init (x);
 
-  check (2, 1000);
+  /* increase the 2nd argument to say 300000 to perform the exhaustive search
+     in src/const_log2.c */
+  check (MPFR_PREC_MIN, 1000);
 
   /* check precision of 2 bits */
   mpfr_set_prec (x, 2);
@@ -164,11 +180,10 @@ main (int argc, char *argv[])
       exit (1);
     }
 
-  if (argc>=2)
+  if (argc >= 2)
     {
       mpfr_set_prec (x, p);
       mpfr_const_log2 (x, rnd);
-      printf ("log(2)=");
       mpfr_out_str (stdout, 10, 0, x, rnd);
       puts ("");
     }
@@ -189,12 +204,12 @@ main (int argc, char *argv[])
       exit (1);
     }
 
-  mpfr_clear(x);
+  mpfr_clear (x);
 
-  check_large();
+  check_large ();
   check_cache ();
 
-  test_generic (2, 200, 1);
+  test_generic (MPFR_PREC_MIN, 200, 1);
 
   tests_end_mpfr ();
   return 0;
