@@ -1,7 +1,7 @@
 /* Test file for mpfr_get_decimal64 and mpfr_set_decimal64.
 
-Copyright 2006, 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
-Contributed by the Arenaire and Cacao projects, INRIA.
+Copyright 2006-2017 Free Software Foundation, Inc.
+Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -20,16 +20,23 @@ along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
 http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
-#include <stdlib.h> /* for exit */
-#include "mpfr-test.h"
-
-/* #define DEBUG */
+#ifdef HAVE_CONFIG_H
+# include "config.h"
+#endif
 
 #ifdef MPFR_WANT_DECIMAL_FLOATS
+
+#include "mpfr-test.h"
+
+#ifndef DEC64_MAX
+# define DEC64_MAX 9.999999999999999E384dd
+#endif
+
+#if _MPFR_IEEE_FLOATS
 static void
 print_decimal64 (_Decimal64 d)
 {
-  union ieee_double_extract x;
+  union mpfr_ieee_double_extract x;
   union ieee_double_decimal64 y;
   unsigned int Gh, i;
 
@@ -47,77 +54,113 @@ print_decimal64 (_Decimal64 d)
     printf ("%d", (x.s.manl >> (i - 1)) & 1);
   printf ("|\n");
 }
+#else
+/* Portable version, assuming long double has at least 55 bits.
+   Note: __STDC_WANT_IEC_60559_DFP_EXT__ or __STDC_WANT_DEC_FP__
+   might allow to use printf("%.15De\n", d) */
+static void
+print_decimal64 (_Decimal64 d)
+{
+  printf ("%.15Le\n", (long double) d);
+}
+#endif /* _MPFR_IEEE_FLOATS */
+
+#define PRINT_ERR_MISC(V)                                   \
+  do                                                        \
+    {                                                       \
+      printf ("Error in check_misc for %s.\n", V);          \
+      printf ("  mpfr_get_decimal64() returned: ");         \
+      print_decimal64 (d);                                  \
+      printf ("  mpfr_set_decimal64() set x to: ");         \
+      mpfr_out_str (stdout, 10, 0, x, MPFR_RNDN);           \
+      printf (" approx.\n    = ");                          \
+      mpfr_dump (x);                                        \
+      err = 1;                                              \
+    }                                                       \
+ while (0)
 
 static void
-check_inf_nan (void)
+check_misc (void)
 {
   mpfr_t  x, y;
   _Decimal64 d;
+  int err = 0;
 
   mpfr_init2 (x, 123);
   mpfr_init2 (y, 123);
 
+#if !defined(MPFR_ERRDIVZERO)
   mpfr_set_nan (x);
   d = mpfr_get_decimal64 (x, MPFR_RNDZ);
   mpfr_set_ui (x, 1, MPFR_RNDZ);
   mpfr_set_decimal64 (x, d, MPFR_RNDZ);
-  ASSERT_ALWAYS (mpfr_nan_p (x));
+  MPFR_ASSERTN (mpfr_nan_p (x));
 
   mpfr_set_inf (x, 1);
   d = mpfr_get_decimal64 (x, MPFR_RNDZ);
   mpfr_set_ui (x, 1, MPFR_RNDZ);
   mpfr_set_decimal64 (x, d, MPFR_RNDZ);
-  ASSERT_ALWAYS (mpfr_inf_p (x) && mpfr_sgn (x) > 0);
+  if (! mpfr_inf_p (x) || MPFR_IS_NEG (x))
+    PRINT_ERR_MISC ("+Inf");
 
   mpfr_set_inf (x, -1);
   d = mpfr_get_decimal64 (x, MPFR_RNDZ);
   mpfr_set_ui (x, 1, MPFR_RNDZ);
   mpfr_set_decimal64 (x, d, MPFR_RNDZ);
-  ASSERT_ALWAYS (mpfr_inf_p (x) && mpfr_sgn (x) < 0);
+  if (! mpfr_inf_p (x) || MPFR_IS_POS (x))
+    PRINT_ERR_MISC ("-Inf");
+#endif
 
   mpfr_set_ui (x, 0, MPFR_RNDZ);
   d = mpfr_get_decimal64 (x, MPFR_RNDZ);
   mpfr_set_ui (x, 1, MPFR_RNDZ);
   mpfr_set_decimal64 (x, d, MPFR_RNDZ);
-  ASSERT_ALWAYS (mpfr_cmp_ui (x, 0) == 0 && MPFR_SIGN (x) > 0);
+  if (MPFR_NOTZERO (x) || MPFR_IS_NEG (x))
+    PRINT_ERR_MISC ("+0");
 
   mpfr_set_ui (x, 0, MPFR_RNDZ);
   mpfr_neg (x, x, MPFR_RNDZ);
   d = mpfr_get_decimal64 (x, MPFR_RNDZ);
   mpfr_set_ui (x, 1, MPFR_RNDZ);
   mpfr_set_decimal64 (x, d, MPFR_RNDZ);
-  ASSERT_ALWAYS (mpfr_cmp_ui (x, 0) == 0 && MPFR_SIGN (x) < 0);
+  if (MPFR_NOTZERO (x) || MPFR_IS_POS (x))
+    PRINT_ERR_MISC ("-0");
 
   mpfr_set_ui (x, 1, MPFR_RNDZ);
   d = mpfr_get_decimal64 (x, MPFR_RNDZ);
   mpfr_set_ui (x, 0, MPFR_RNDZ);
   mpfr_set_decimal64 (x, d, MPFR_RNDZ);
-  ASSERT_ALWAYS (mpfr_cmp_ui (x, 1) == 0);
+  if (mpfr_cmp_ui (x, 1) != 0)
+    PRINT_ERR_MISC ("+1");
 
   mpfr_set_si (x, -1, MPFR_RNDZ);
   d = mpfr_get_decimal64 (x, MPFR_RNDZ);
   mpfr_set_ui (x, 0, MPFR_RNDZ);
   mpfr_set_decimal64 (x, d, MPFR_RNDZ);
-  ASSERT_ALWAYS (mpfr_cmp_si (x, -1) == 0);
+  if (mpfr_cmp_si (x, -1) != 0)
+    PRINT_ERR_MISC ("-1");
 
   mpfr_set_ui (x, 2, MPFR_RNDZ);
   d = mpfr_get_decimal64 (x, MPFR_RNDZ);
   mpfr_set_ui (x, 0, MPFR_RNDZ);
   mpfr_set_decimal64 (x, d, MPFR_RNDZ);
-  ASSERT_ALWAYS (mpfr_cmp_ui (x, 2) == 0);
+  if (mpfr_cmp_ui (x, 2) != 0)
+    PRINT_ERR_MISC ("2");
 
   mpfr_set_ui (x, 99, MPFR_RNDZ);
   d = mpfr_get_decimal64 (x, MPFR_RNDZ);
   mpfr_set_ui (x, 0, MPFR_RNDZ);
   mpfr_set_decimal64 (x, d, MPFR_RNDZ);
-  ASSERT_ALWAYS (mpfr_cmp_ui (x, 99) == 0);
+  if (mpfr_cmp_ui (x, 99) != 0)
+    PRINT_ERR_MISC ("99");
 
   mpfr_set_str (x, "9999999999999999", 10, MPFR_RNDZ);
   mpfr_set (y, x, MPFR_RNDZ);
   d = mpfr_get_decimal64 (x, MPFR_RNDZ);
   mpfr_set_ui (x, 0, MPFR_RNDZ);
   mpfr_set_decimal64 (x, d, MPFR_RNDZ);
-  ASSERT_ALWAYS (mpfr_cmp (x, y) == 0);
+  if (! mpfr_equal_p (x, y))
+    PRINT_ERR_MISC ("9999999999999999");
 
   /* smallest normal number */
   mpfr_set_str (x, "1E-383", 10, MPFR_RNDU);
@@ -125,7 +168,8 @@ check_inf_nan (void)
   d = mpfr_get_decimal64 (x, MPFR_RNDZ);
   mpfr_set_ui (x, 0, MPFR_RNDZ);
   mpfr_set_decimal64 (x, d, MPFR_RNDU);
-  ASSERT_ALWAYS (mpfr_cmp (x, y) == 0);
+  if (! mpfr_equal_p (x, y))
+    PRINT_ERR_MISC ("1E-383");
 
   /* smallest subnormal number */
   mpfr_set_str (x, "1E-398", 10, MPFR_RNDU);
@@ -133,7 +177,8 @@ check_inf_nan (void)
   d = mpfr_get_decimal64 (x, MPFR_RNDZ);
   mpfr_set_ui (x, 0, MPFR_RNDZ);
   mpfr_set_decimal64 (x, d, MPFR_RNDU);
-  ASSERT_ALWAYS (mpfr_cmp (x, y) == 0);
+  if (! mpfr_equal_p (x, y))
+    PRINT_ERR_MISC ("1E-398");
 
   /* subnormal number with exponent change when we round back
      from 16 digits to 1 digit */
@@ -142,15 +187,45 @@ check_inf_nan (void)
   mpfr_set_ui (x, 0, MPFR_RNDZ);
   mpfr_set_decimal64 (x, d, MPFR_RNDD);
   mpfr_set_str (y, "1E-397", 10, MPFR_RNDN);
-  ASSERT_ALWAYS (mpfr_cmp (x, y) == 0);
+  if (! mpfr_equal_p (x, y))
+    PRINT_ERR_MISC ("9.9E-398");
 
   /* largest number */
   mpfr_set_str (x, "9.999999999999999E384", 10, MPFR_RNDZ);
   mpfr_set (y, x, MPFR_RNDZ);
   d = mpfr_get_decimal64 (x, MPFR_RNDU);
-  mpfr_set_ui (x, 0, MPFR_RNDZ);
-  mpfr_set_decimal64 (x, d, MPFR_RNDZ);
-  ASSERT_ALWAYS (mpfr_cmp (x, y) == 0);
+  if (d == DEC64_MAX)
+    {
+      mpfr_set_ui (x, 0, MPFR_RNDZ);
+      mpfr_set_decimal64 (x, d, MPFR_RNDZ);
+      if (! mpfr_equal_p (x, y))
+        PRINT_ERR_MISC ("DEC64_MAX");
+    }
+  else
+    {
+      printf ("Error in check_misc for DEC64_MAX.\n");
+      printf ("  mpfr_get_decimal64() returned: ");
+      print_decimal64 (d);
+      err = 1;
+    }
+
+  mpfr_set_str (x, "-9.999999999999999E384", 10, MPFR_RNDZ);
+  mpfr_set (y, x, MPFR_RNDZ);
+  d = mpfr_get_decimal64 (x, MPFR_RNDA);
+  if (d == -DEC64_MAX)
+    {
+      mpfr_set_ui (x, 0, MPFR_RNDZ);
+      mpfr_set_decimal64 (x, d, MPFR_RNDZ);
+      if (! mpfr_equal_p (x, y))
+        PRINT_ERR_MISC ("-DEC64_MAX");
+    }
+  else
+    {
+      printf ("Error in check_misc for -DEC64_MAX.\n");
+      printf ("  mpfr_get_decimal64() returned: ");
+      print_decimal64 (d);
+      err = 1;
+    }
 
   mpfr_set_prec (x, 53);
   mpfr_set_prec (y, 53);
@@ -159,10 +234,14 @@ check_inf_nan (void)
   mpfr_set_str (x, "9.999999999999999E384", 10, MPFR_RNDZ);
   d = mpfr_get_decimal64 (x, MPFR_RNDZ);
   mpfr_set_decimal64 (y, d, MPFR_RNDU);
-  ASSERT_ALWAYS (mpfr_cmp (x, y) == 0);
+  if (! mpfr_equal_p (x, y))
+    PRINT_ERR_MISC ("DEC64_MAX (2)");
 
   mpfr_clear (x);
   mpfr_clear (y);
+
+  if (err)
+    exit (1);
 }
 
 static void
@@ -186,6 +265,7 @@ check_random (void)
       mpfr_set_decimal64 (y, d, MPFR_RNDN);
       if (mpfr_cmp (x, y) != 0)
         {
+          printf ("Error:\n");
           printf ("x="); mpfr_dump (x);
           printf ("d="); print_decimal64 (d);
           printf ("y="); mpfr_dump (y);
@@ -223,7 +303,83 @@ check_native (void)
 
   mpfr_clear (x);
 }
-#endif /* MPFR_WANT_DECIMAL_FLOATS */
+
+static void
+check_overflow (void)
+{
+  mpfr_t x;
+  int err = 0, neg, rnd;
+
+  mpfr_init2 (x, 96);
+  for (neg = 0; neg < 2; neg++)
+    RND_LOOP (rnd)
+      {
+        _Decimal64 d, e;
+        mpfr_rnd_t r = (mpfr_rnd_t) rnd;
+        int sign = neg ? -1 : 1;
+
+        e = sign * (MPFR_IS_LIKE_RNDZ (r, neg) ? 1 : 2) * DEC64_MAX;
+        /* This tests the binary exponent e > 1279 case of get_d64.c */
+        mpfr_set_si_2exp (x, sign, 9999, MPFR_RNDN);
+        d = mpfr_get_decimal64 (x, r);
+        if (d != e)
+          {
+            printf ("Error 1 in check_overflow for %s, %s\n",
+                    neg ? "negative" : "positive",
+                    mpfr_print_rnd_mode (r));
+            err = 1;
+          }
+        /* This tests the decimal exponent e > 385 case of get_d64.c */
+        mpfr_set_si_2exp (x, sign * 31, 1274, MPFR_RNDN);
+        d = mpfr_get_decimal64 (x, r);
+        if (d != e)
+          {
+            printf ("Error 2 in check_overflow for %s, %s\n",
+                    neg ? "negative" : "positive",
+                    mpfr_print_rnd_mode (r));
+            err = 1;
+          }
+        /* This tests the last else (-382 <= e <= 385) of get_d64.c */
+        mpfr_set_decimal64 (x, e, MPFR_RNDA);
+        d = mpfr_get_decimal64 (x, r);
+        if (d != e)
+          {
+            printf ("Error 3 in check_overflow for %s, %s\n",
+                    neg ? "negative" : "positive",
+                    mpfr_print_rnd_mode (r));
+            err = 1;
+          }
+      }
+  mpfr_clear (x);
+  if (err)
+    exit (1);
+}
+
+static void
+check_tiny (void)
+{
+  mpfr_t x;
+  _Decimal64 d;
+
+  /* If 0.5E-398 < |x| < 1E-398 (smallest subnormal), x should round
+     to +/- 1E-398 in MPFR_RNDN. Note: the midpoint 0.5E-398 between
+     0 and 1E-398 is not a representable binary number, so that there
+     are no tests for it. */
+  mpfr_init2 (x, 128);
+  mpfr_set_str (x, "1E-398", 10, MPFR_RNDZ);
+  d = mpfr_get_decimal64 (x, MPFR_RNDN);
+  MPFR_ASSERTN (d == 1.0E-398dd);
+  mpfr_neg (x, x, MPFR_RNDN);
+  d = mpfr_get_decimal64 (x, MPFR_RNDN);
+  MPFR_ASSERTN (d == -1.0E-398dd);
+  mpfr_set_str (x, "0.5E-398", 10, MPFR_RNDU);
+  d = mpfr_get_decimal64 (x, MPFR_RNDN);
+  MPFR_ASSERTN (d == 1.0E-398dd);
+  mpfr_neg (x, x, MPFR_RNDN);
+  d = mpfr_get_decimal64 (x, MPFR_RNDN);
+  MPFR_ASSERTN (d == -1.0E-398dd);
+  mpfr_clear (x);
+}
 
 int
 main (void)
@@ -231,19 +387,31 @@ main (void)
   tests_start_mpfr ();
   mpfr_test_init ();
 
-#ifdef MPFR_WANT_DECIMAL_FLOATS
-#ifdef DEBUG
+#ifdef MPFR_DEBUG
 #ifdef DPD_FORMAT
   printf ("Using DPD format\n");
 #else
   printf ("Using BID format\n");
 #endif
 #endif
-  check_inf_nan ();
+  check_misc ();
   check_random ();
   check_native ();
+#if !defined(MPFR_ERRDIVZERO)
+  check_overflow ();
 #endif
+  check_tiny ();
 
   tests_end_mpfr ();
   return 0;
 }
+
+#else /* MPFR_WANT_DECIMAL_FLOATS */
+
+int
+main (void)
+{
+  return 77;
+}
+
+#endif /* MPFR_WANT_DECIMAL_FLOATS */
