@@ -1,7 +1,7 @@
 /* Test file for mpfr_sqrt.
 
-Copyright 1999, 2001, 2002, 2003, 2004, 2005, 2006, 2007, 2008, 2009, 2010, 2011 Free Software Foundation, Inc.
-Contributed by the Arenaire and Cacao projects, INRIA.
+Copyright 1999, 2001-2017 Free Software Foundation, Inc.
+Contributed by the AriC and Caramba projects, INRIA.
 
 This file is part of the GNU MPFR Library.
 
@@ -20,9 +20,6 @@ along with the GNU MPFR Library; see the file COPYING.LESSER.  If not, see
 http://www.gnu.org/licenses/ or write to the Free Software Foundation, Inc.,
 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA. */
 
-#include <stdio.h>
-#include <stdlib.h>
-
 #include "mpfr-test.h"
 
 #ifdef CHECK_EXTERNAL
@@ -69,20 +66,20 @@ check3 (const char *as, mpfr_rnd_t rnd_mode, const char *qs)
 }
 
 static void
-check4 (const char *as, mpfr_rnd_t rnd_mode, const char *Qs)
+check4 (const char *as, mpfr_rnd_t rnd_mode, const char *qs)
 {
   mpfr_t q;
 
   mpfr_init2 (q, 53);
   mpfr_set_str1 (q, as);
   test_sqrt (q, q, rnd_mode);
-  if (mpfr_cmp_str (q, Qs, 16, MPFR_RNDN))
+  if (mpfr_cmp_str (q, qs, 16, MPFR_RNDN))
     {
       printf ("mpfr_sqrt failed for a=%s, rnd_mode=%s\n",
-              as, mpfr_print_rnd_mode(rnd_mode));
-      printf ("expected ");
+              as, mpfr_print_rnd_mode (rnd_mode));
+      printf ("expected %s\ngot      ", qs);
       mpfr_out_str (stdout, 16, 0, q, MPFR_RNDN);
-      printf ("\ngot      %s\n", Qs);
+      printf ("\n");
       mpfr_clear (q);
       exit (1);
     }
@@ -120,7 +117,7 @@ check_diverse (const char *as, mpfr_prec_t p, const char *qs)
   if (mpfr_cmp_str1 (q, qs))
     {
       printf ("mpfr_sqrt failed for a=%s, prec=%lu, rnd_mode=%s\n",
-              as, p, mpfr_print_rnd_mode (MPFR_RNDN));
+              as, (unsigned long) p, mpfr_print_rnd_mode (MPFR_RNDN));
       printf ("expected sqrt is %s, got ", qs);
       mpfr_out_str (stdout, 10, 0, q, MPFR_RNDN);
       printf ("\n");
@@ -255,7 +252,7 @@ special (void)
         {
           printf ("Error: sqrt(1+ulp(1), up) should give 1.5 (prec=%u)\n",
                   (unsigned int) p);
-          printf ("got "); mpfr_print_binary (x); puts ("");
+          printf ("got "); mpfr_dump (x);
           exit (1);
         }
     }
@@ -279,8 +276,7 @@ special (void)
   if (mpfr_cmp_ui (z, 0) < 0)
     {
       printf ("Error: square root of 1 gives ");
-      mpfr_print_binary(z);
-      putchar('\n');
+      mpfr_dump (z);
       exit (1);
     }
 
@@ -323,7 +319,7 @@ special (void)
 
   /* case prec(result) << prec(input) */
   mpfr_set_prec (z, 2);
-  for (p = 2; p < 1000; p++)
+  for (p = mpfr_get_prec (z); p < 1000; p++)
     {
       mpfr_set_prec (x, p);
       mpfr_set_ui (x, 1, MPFR_RNDN);
@@ -354,11 +350,13 @@ special (void)
       mpfr_set_exp (x, GMP_NUMB_BITS);
       mpfr_add_ui (x, x, 1, MPFR_RNDN);
       /* now x = 2^(GMP_NUMB_BITS - 1) + 1 (GMP_NUMB_BITS bits) */
-      MPFR_ASSERTN (mpfr_mul (x, x, x, MPFR_RNDN) == 0); /* exact */
+      inexact = mpfr_mul (x, x, x, MPFR_RNDN);
+      MPFR_ASSERTN (inexact == 0); /* exact */
       inexact = test_sqrt (z, x, MPFR_RNDN);
       /* even rule: z should be 2^(GMP_NUMB_BITS - 1) */
       MPFR_ASSERTN (inexact < 0);
-      MPFR_ASSERTN (mpfr_cmp_ui_2exp (z, 1, GMP_NUMB_BITS - 1) == 0);
+      inexact = mpfr_cmp_ui_2exp (z, 1, GMP_NUMB_BITS - 1);
+      MPFR_ASSERTN (inexact == 0);
       mpfr_nextbelow (x);
       /* now x is just below [2^(GMP_NUMB_BITS - 1) + 1]^2 */
       inexact = test_sqrt (z, x, MPFR_RNDN);
@@ -429,7 +427,7 @@ check_inexact (mpfr_prec_t p)
   mpfr_init2 (y, p);
   mpfr_init2 (z, 2*p);
   mpfr_urandomb (x, RANDS);
-  rnd = RND_RAND ();
+  rnd = RND_RAND_NO_RNDF ();
   inexact = test_sqrt (y, x, rnd);
   if (mpfr_mul (z, y, y, rnd)) /* exact since prec(z) = 2*prec(y) */
     {
@@ -442,12 +440,12 @@ check_inexact (mpfr_prec_t p)
       ((inexact > 0) && (sign <= 0)) ||
       ((inexact < 0) && (sign >= 0)))
     {
-      printf ("Error: wrong inexact flag, expected %d, got %d\n",
-              sign, inexact);
+      printf ("Error with rnd=%s: wrong ternary value, expected %d, got %d\n",
+              mpfr_print_rnd_mode (rnd), sign, inexact);
       printf ("x=");
-      mpfr_print_binary (x);
-      printf (" rnd=%s\n", mpfr_print_rnd_mode (rnd));
-      printf ("y="); mpfr_print_binary (y); puts ("");
+      mpfr_dump (x);
+      printf ("y=");
+      mpfr_dump (y);
       exit (1);
     }
   mpfr_clear (x);
@@ -456,7 +454,7 @@ check_inexact (mpfr_prec_t p)
 }
 
 static void
-check_nan (void)
+check_singular (void)
 {
   mpfr_t  x, got;
 
@@ -485,12 +483,20 @@ check_nan (void)
   MPFR_ASSERTN (test_sqrt (got, x, MPFR_RNDZ) == 0); /* exact */
   MPFR_ASSERTN (mpfr_nan_p (got));
 
-  /* sqrt(-0) == 0 */
+  /* sqrt(+0) == +0 */
+  mpfr_set_si (x, 0L, MPFR_RNDZ);
+  MPFR_ASSERTN (test_sqrt (got, x, MPFR_RNDZ) == 0); /* exact */
+  MPFR_ASSERTN (mpfr_number_p (got));
+  MPFR_ASSERTN (mpfr_cmp_ui (got, 0L) == 0);
+  MPFR_ASSERTN (MPFR_IS_POS (got));
+
+  /* sqrt(-0) == -0 */
   mpfr_set_si (x, 0L, MPFR_RNDZ);
   MPFR_SET_NEG (x);
   MPFR_ASSERTN (test_sqrt (got, x, MPFR_RNDZ) == 0); /* exact */
   MPFR_ASSERTN (mpfr_number_p (got));
   MPFR_ASSERTN (mpfr_cmp_ui (got, 0L) == 0);
+  MPFR_ASSERTN (MPFR_IS_NEG (got));
 
   mpfr_clear (x);
   mpfr_clear (got);
@@ -561,6 +567,128 @@ test_property2 (mpfr_prec_t p, mpfr_rnd_t r)
   mpfr_clear (y);
 }
 
+/* Bug reported by Fredrik Johansson, occurring when:
+   - the precision of the result is a multiple of the number of bits
+     per word (GMP_NUMB_BITS),
+   - the rounding mode is to nearest (MPFR_RNDN),
+   - internally, the result has to be rounded up to a power of 2.
+*/
+static void
+bug20160120 (void)
+{
+  mpfr_t x, y;
+
+  mpfr_init2 (x, 4 * GMP_NUMB_BITS);
+  mpfr_init2 (y, GMP_NUMB_BITS);
+
+  mpfr_set_ui (x, 1, MPFR_RNDN);
+  mpfr_nextbelow (x);
+  mpfr_sqrt (y, x, MPFR_RNDN);
+  MPFR_ASSERTN(mpfr_check (y));
+  MPFR_ASSERTN(mpfr_cmp_ui (y, 1) == 0);
+
+  mpfr_set_prec (y, 2 * GMP_NUMB_BITS);
+  mpfr_sqrt (y, x, MPFR_RNDN);
+  MPFR_ASSERTN(mpfr_check (y));
+  MPFR_ASSERTN(mpfr_cmp_ui (y, 1) == 0);
+
+  mpfr_clear(x);
+  mpfr_clear(y);
+}
+
+/* Bug in mpfr_sqrt2 when prec(u) = 2*GMP_NUMB_BITS and the exponent of u is
+   odd: the last bit of u is lost. */
+static void
+bug20160908 (void)
+{
+  mpfr_t r, u;
+  int n = GMP_NUMB_BITS, ret;
+
+  mpfr_init2 (r, 2*n - 1);
+  mpfr_init2 (u, 2 * n);
+  mpfr_set_ui_2exp (u, 1, 2*n-2, MPFR_RNDN); /* u=2^(2n-2) with exp(u)=2n-1 */
+  mpfr_nextabove (u);
+  /* now u = 2^(2n-2) + 1/2 */
+  ret = mpfr_sqrt (r, u, MPFR_RNDZ);
+  MPFR_ASSERTN(ret == -1 && mpfr_cmp_ui_2exp (r, 1, n-1) == 0);
+  mpfr_clear (r);
+  mpfr_clear (u);
+}
+
+static void
+testall_rndf (mpfr_prec_t pmax)
+{
+  mpfr_t a, b, d;
+  mpfr_prec_t pa, pb;
+
+  for (pa = MPFR_PREC_MIN; pa <= pmax; pa++)
+    {
+      mpfr_init2 (a, pa);
+      mpfr_init2 (d, pa);
+      for (pb = MPFR_PREC_MIN; pb <= pmax; pb++)
+        {
+          mpfr_init2 (b, pb);
+          mpfr_set_ui (b, 1, MPFR_RNDN);
+          while (mpfr_cmp_ui (b, 4) < 0)
+            {
+              mpfr_sqrt (a, b, MPFR_RNDF);
+              mpfr_sqrt (d, b, MPFR_RNDD);
+              if (!mpfr_equal_p (a, d))
+                {
+                  mpfr_sqrt (d, b, MPFR_RNDU);
+                  if (!mpfr_equal_p (a, d))
+                    {
+                      printf ("Error: mpfr_sqrt(a,b,RNDF) does not "
+                              "match RNDD/RNDU\n");
+                      printf ("b="); mpfr_dump (b);
+                      printf ("a="); mpfr_dump (a);
+                      exit (1);
+                    }
+                }
+              mpfr_nextabove (b);
+            }
+          mpfr_clear (b);
+        }
+      mpfr_clear (a);
+      mpfr_clear (d);
+    }
+}
+
+/* test the case prec = GMP_NUMB_BITS */
+static void
+test_sqrt1n (void)
+{
+  mpfr_t r, u;
+  int inex;
+
+  mpfr_init2 (r, GMP_NUMB_BITS);
+  mpfr_init2 (u, GMP_NUMB_BITS);
+
+  inex = mpfr_set_ui_2exp (u, 17 * 17, 2 * GMP_NUMB_BITS - 10, MPFR_RNDN);
+  MPFR_ASSERTN(inex == 0);
+  inex = mpfr_sqrt (r, u, MPFR_RNDN);
+  MPFR_ASSERTN(inex == 0);
+  MPFR_ASSERTN(mpfr_cmp_ui_2exp (r, 17, GMP_NUMB_BITS - 5) == 0);
+
+  inex = mpfr_set_ui_2exp (u, 1, GMP_NUMB_BITS - 2, MPFR_RNDN);
+  MPFR_ASSERTN(inex == 0);
+  inex = mpfr_add_ui (u, u, 1, MPFR_RNDN);
+  MPFR_ASSERTN(inex == 0);
+  inex = mpfr_mul_2exp (u, u, GMP_NUMB_BITS, MPFR_RNDN);
+  MPFR_ASSERTN(inex == 0);
+  /* u = 2^(2*GMP_NUMB_BITS-2) + 2^GMP_NUMB_BITS, thus
+     u = r^2 + 2^GMP_NUMB_BITS with r = 2^(GMP_NUMB_BITS-1).
+     Should round to r+1 to nearest. */
+  inex = mpfr_sqrt (r, u, MPFR_RNDN);
+  MPFR_ASSERTN(inex > 0);
+  inex = mpfr_sub_ui (r, r, 1, MPFR_RNDN);
+  MPFR_ASSERTN(inex == 0);
+  MPFR_ASSERTN(mpfr_cmp_ui_2exp (r, 1, GMP_NUMB_BITS - 1) == 0);
+
+  mpfr_clear (r);
+  mpfr_clear (u);
+}
+
 #define TEST_FUNCTION test_sqrt
 #define TEST_RANDOM_POS 8
 #include "tgeneric.c"
@@ -573,6 +701,7 @@ main (void)
 
   tests_start_mpfr ();
 
+  testall_rndf (16);
   for (p = MPFR_PREC_MIN; p <= 128; p++)
     {
       test_property1 (p, MPFR_RNDN, 0);
@@ -586,7 +715,7 @@ main (void)
 
   check_diverse ("635030154261163106768013773815762607450069292760790610550915652722277604820131530404842415587328", 160, "796887792767063979679855997149887366668464780637");
   special ();
-  check_nan ();
+  check_singular ();
 
   for (p=2; p<200; p++)
     for (k=0; k<200; k++)
@@ -692,10 +821,14 @@ main (void)
   check4 ("72154663483843080704304789585920.0", MPFR_RNDA,
           "1.e2d9a51977e6e@13");
 
-  test_generic (2, 300, 15);
+  test_generic (MPFR_PREC_MIN, 300, 15);
   data_check ("data/sqrt", mpfr_sqrt, "mpfr_sqrt");
   bad_cases (mpfr_sqrt, mpfr_sqr, "mpfr_sqrt", 8, -256, 255, 4, 128, 800, 50);
 
+  bug20160120 ();
+  bug20160908 ();
+  test_sqrt1n ();
+
   tests_end_mpfr ();
   return 0;
 }
